@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -46,7 +46,6 @@ void gesv_checkBadArgs(const hipsolverHandle_t handle,
                        U                       dInfo,
                        const int               bc)
 {
-#if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // handle
     EXPECT_ROCBLAS_STATUS(hipsolver_gesv(API,
                                          false,
@@ -74,6 +73,7 @@ void gesv_checkBadArgs(const hipsolverHandle_t handle,
     // values
     // N/A
 
+#if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // pointers
     EXPECT_ROCBLAS_STATUS(hipsolver_gesv(API,
                                          false,
@@ -232,7 +232,7 @@ void testing_gesv_bad_arg()
         //                           dX.data(),
         //                           ldx,
         //                           &size_W);
-        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         // if(size_W)
         //     CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -285,7 +285,7 @@ void testing_gesv_bad_arg()
                                   dX.data(),
                                   ldx,
                                   &size_W);
-        device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         if(size_W)
             CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -429,7 +429,7 @@ void gesv_getError(const hipsolverHandle_t handle,
     // CPU lapack
     for(int b = 0; b < bc; ++b)
     {
-        cblas_gesv<T>(n, nrhs, hA[b], lda, hIpiv[b], hB[b], ldb, hInfo[b]);
+        cpu_gesv(n, nrhs, hA[b], lda, hIpiv[b], hB[b], ldb, hInfo[b]);
     }
 
     // error is ||hB - hBRes|| / ||hB||
@@ -450,8 +450,11 @@ void gesv_getError(const hipsolverHandle_t handle,
     // also check info for singularities
     err = 0;
     for(int b = 0; b < bc; ++b)
+    {
+        EXPECT_EQ(hInfo[b][0], hInfoRes[b][0]) << "where b = " << b;
         if(hInfo[b][0] != hInfoRes[b][0])
             err++;
+    }
     *max_err += err;
 }
 
@@ -499,7 +502,7 @@ void gesv_getPerfData(const hipsolverHandle_t handle,
         *cpu_time_used = get_time_us_no_sync();
         for(int b = 0; b < bc; ++b)
         {
-            cblas_gesv<T>(n, nrhs, hA[b], lda, hIpiv[b], hB[b], ldb, hInfo[b]);
+            cpu_gesv(n, nrhs, hA[b], lda, hIpiv[b], hB[b], ldb, hInfo[b]);
         }
         *cpu_time_used = get_time_us_no_sync() - *cpu_time_used;
     }
@@ -702,7 +705,7 @@ void testing_gesv(Arguments& argus)
         // device_batch_vector<T>           dX(size_X, 1, bc);
         // device_strided_batch_vector<int> dIpiv(size_P, 1, stP, bc);
         // device_strided_batch_vector<int> dInfo(1, 1, 1, bc);
-        // device_strided_batch_vector<T>   dWork(size_W, 1, size_W, bc);
+        // device_strided_batch_vector<T>   dWork(size_W, 1, size_W, 1); // size_W accounts for bc
         // if(size_A)
         //     CHECK_HIP_ERROR(dA.memcheck());
         // if(size_B)
@@ -790,7 +793,7 @@ void testing_gesv(Arguments& argus)
         device_strided_batch_vector<T>   dX(size_X, 1, stX, bc);
         device_strided_batch_vector<int> dIpiv(size_P, 1, stP, bc);
         device_strided_batch_vector<int> dInfo(1, 1, 1, bc);
-        device_strided_batch_vector<T>   dWork(size_W, 1, size_W, bc);
+        device_strided_batch_vector<T>   dWork(size_W, 1, size_W, 1); // size_W accounts for bc
         if(size_A)
             CHECK_HIP_ERROR(dA.memcheck());
         if(size_B)
