@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -48,7 +48,6 @@ void gesvda_checkBadArgs(const hipsolverHandle_t handle,
                          double*                 hRnrmF,
                          const int               bc)
 {
-#if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // handle
     EXPECT_ROCBLAS_STATUS(hipsolver_gesvda(API,
                                            STRIDED,
@@ -101,6 +100,7 @@ void gesvda_checkBadArgs(const hipsolverHandle_t handle,
                                            bc),
                           HIPSOLVER_STATUS_INVALID_ENUM);
 
+#if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // pointers
     EXPECT_ROCBLAS_STATUS(hipsolver_gesvda(API,
                                            STRIDED,
@@ -281,7 +281,7 @@ void testing_gesvda_bad_arg()
         //                             stV,
         //                             &size_W,
         //                             bc);
-        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         // if(size_W)
         //     CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -344,7 +344,7 @@ void testing_gesvda_bad_arg()
                                     stV,
                                     &size_W,
                                     bc);
-        device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         if(size_W)
             CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -520,27 +520,30 @@ void gesvda_getError(const hipsolverHandle_t handle,
     // CPU lapack
     // Only singular values needed
     for(int b = 0; b < bc; ++b)
-        cblas_gesvd<T>('N',
-                       'N',
-                       m,
-                       n,
-                       hA[b],
-                       lda,
-                       hS[b],
-                       nullptr,
-                       ldu,
-                       nullptr,
-                       ldv,
-                       hWork.data(),
-                       size_W,
-                       hE.data(),
-                       hinfo[b]);
+        cpu_gesvd<T>('N',
+                     'N',
+                     m,
+                     n,
+                     hA[b],
+                     lda,
+                     hS[b],
+                     nullptr,
+                     ldu,
+                     nullptr,
+                     ldv,
+                     hWork.data(),
+                     size_W,
+                     hE.data(),
+                     hinfo[b]);
 
     // // Check info for non-convergence
     *max_err = 0;
     // for(int b = 0; b < bc; ++b)
+    // {
+    //     EXPECT_EQ(hinfo[b][0], hinfoRes[b][0]) << "where b = " << b;
     //     if(hinfo[b][0] != hinfoRes[b][0])
     //         *max_err += 1;
+    // }
 
     double err;
     *max_errv = 0;
@@ -714,9 +717,9 @@ void testing_gesvda(Arguments& argus)
     int                    ldu   = argus.get<int>("ldu", m);
     int                    ldv   = argus.get<int>("ldv", n);
     rocblas_stride         stA   = argus.get<rocblas_stride>("strideA", lda * n);
-    rocblas_stride         stS   = argus.get<rocblas_stride>("strideS", min(m, n));
-    rocblas_stride         stU   = argus.get<rocblas_stride>("strideU", ldu * min(m, n));
-    rocblas_stride         stV   = argus.get<rocblas_stride>("strideV", ldv * min(m, n));
+    rocblas_stride         stS   = argus.get<rocblas_stride>("strideS", rank);
+    rocblas_stride         stU   = argus.get<rocblas_stride>("strideU", ldu * rank);
+    rocblas_stride         stV   = argus.get<rocblas_stride>("strideV", ldv * rank);
 
     hipsolverEigMode_t jobz      = char2hipsolver_evect(jobzC);
     int                bc        = argus.batch_count;
@@ -726,10 +729,11 @@ void testing_gesvda(Arguments& argus)
     rocblas_stride stVres = 0;
 
     // determine sizes
-    size_t size_A = size_t(lda) * n;
-    size_t size_S = size_t(min(m, n));
-    size_t size_V = 0;
-    size_t size_U = 0;
+    size_t size_A     = size_t(lda) * n;
+    size_t size_S     = size_t(rank);
+    size_t size_S_cpu = size_t(min(m, n));
+    size_t size_V     = 0;
+    size_t size_U     = 0;
 
     size_t size_Sres  = 0;
     size_t size_hUres = 0;
@@ -737,8 +741,8 @@ void testing_gesvda(Arguments& argus)
 
     if(jobz != HIPSOLVER_EIG_MODE_NOVECTOR)
     {
-        size_U = size_t(ldu) * min(m, n);
-        size_V = size_t(ldv) * min(m, n);
+        size_U = size_t(ldu) * rank;
+        size_V = size_t(ldv) * rank;
     }
 
     if(argus.unit_check || argus.norm_check)
@@ -851,7 +855,8 @@ void testing_gesvda(Arguments& argus)
 
     // memory allocations (all cases)
     // host
-    host_strided_batch_vector<S>      hS(size_S, 1, stS, bc);
+    host_strided_batch_vector<S> hS(
+        size_S_cpu, 1, size_S_cpu, bc); // extra space for cpu_gesvd call
     host_strided_batch_vector<T>      hV(size_V, 1, stV, bc);
     host_strided_batch_vector<T>      hU(size_U, 1, stU, bc);
     host_strided_batch_vector<double> hRnrmF(1, 1, 1, bc);
@@ -865,7 +870,7 @@ void testing_gesvda(Arguments& argus)
     device_strided_batch_vector<T>   dV(size_V, 1, stV, bc);
     device_strided_batch_vector<T>   dU(size_U, 1, stU, bc);
     device_strided_batch_vector<int> dinfo(1, 1, 1, bc);
-    device_strided_batch_vector<T>   dWork(size_W, 1, size_W, bc);
+    device_strided_batch_vector<T>   dWork(size_W, 1, size_W, 1); // size_W accounts for bc
     if(size_S)
         CHECK_HIP_ERROR(dS.memcheck());
     if(size_V)
