@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -48,7 +48,6 @@ void gesvdj_checkBadArgs(const hipsolverHandle_t     handle,
                          const hipsolverGesvdjInfo_t params,
                          const int                   bc)
 {
-#if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // handle
     EXPECT_ROCBLAS_STATUS(hipsolver_gesvdj(API,
                                            STRIDED,
@@ -101,6 +100,7 @@ void gesvdj_checkBadArgs(const hipsolverHandle_t     handle,
                                            bc),
                           HIPSOLVER_STATUS_INVALID_ENUM);
 
+#if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // pointers
     EXPECT_ROCBLAS_STATUS(hipsolver_gesvdj(API,
                                            STRIDED,
@@ -278,7 +278,7 @@ void testing_gesvdj_bad_arg()
         //                             &size_W,
         //                             params,
         //                             bc);
-        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         // if(size_W)
         //     CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -337,7 +337,7 @@ void testing_gesvdj_bad_arg()
                                     &size_W,
                                     params,
                                     bc);
-        device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         if(size_W)
             CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -513,39 +513,49 @@ void gesvdj_getError(const hipsolverHandle_t handle,
     // CPU lapack
     // Only singular values needed
     for(int b = 0; b < bc; ++b)
-        cblas_gesvd<T>('N',
-                       'N',
-                       m,
-                       n,
-                       hA[b],
-                       lda,
-                       hS[b],
-                       nullptr,
-                       ldu,
-                       nullptr,
-                       ldv,
-                       hWork.data(),
-                       size_W,
-                       hE.data(),
-                       hinfo[b]);
+        cpu_gesvd<T>('N',
+                     'N',
+                     m,
+                     n,
+                     hA[b],
+                     lda,
+                     hS[b],
+                     nullptr,
+                     ldu,
+                     nullptr,
+                     ldv,
+                     hWork.data(),
+                     size_W,
+                     hE.data(),
+                     hinfo[b]);
 
     // Check info for non-convergence
     *max_err = 0;
     for(int b = 0; b < bc; ++b)
+    {
+        EXPECT_EQ(hinfo[b][0], hinfoRes[b][0]) << "where b = " << b;
         if(hinfo[b][0] != hinfoRes[b][0])
             *max_err += 1;
+    }
 
     if(!STRIDED)
     {
         // Also check validity of residual
         for(rocblas_int b = 0; b < bc; ++b)
+        {
+            EXPECT_GE(hResidualRes[b][0], 0) << "where b = " << b;
             if(hResidualRes[b][0] < 0)
                 *max_err += 1;
+        }
 
         // Also check validity of sweeps
         for(rocblas_int b = 0; b < bc; ++b)
+        {
+            EXPECT_GE(hResidualRes[b][0], 0) << "where b = " << b;
+            EXPECT_LE(hSweepsRes[b][0], max_sweeps) << "where b = " << b;
             if(hSweepsRes[b][0] < 0 || hSweepsRes[b][0] > max_sweeps)
                 *max_err += 1;
+        }
     }
 
     // (We expect the used input matrices to always converge. Testing
@@ -643,21 +653,21 @@ void gesvdj_getPerfData(const hipsolverHandle_t handle,
         // cpu-lapack performance (only if not in perf mode)
         *cpu_time_used = get_time_us_no_sync();
         for(int b = 0; b < bc; ++b)
-            cblas_gesvd<T>(svect,
-                           svect,
-                           m,
-                           n,
-                           hA[b],
-                           lda,
-                           hS[b],
-                           hU[b],
-                           ldu,
-                           hV[b],
-                           ldv_trans,
-                           hWork.data(),
-                           size_W,
-                           hE.data(),
-                           hinfo[b]);
+            cpu_gesvd(svect,
+                      svect,
+                      m,
+                      n,
+                      hA[b],
+                      lda,
+                      hS[b],
+                      hU[b],
+                      ldu,
+                      hV[b],
+                      ldv_trans,
+                      hWork.data(),
+                      size_W,
+                      hE.data(),
+                      hinfo[b]);
         *cpu_time_used = get_time_us_no_sync() - *cpu_time_used;
     }
 
@@ -902,7 +912,7 @@ void testing_gesvdj(Arguments& argus)
     device_strided_batch_vector<T>   dV(size_V, 1, stV, bc);
     device_strided_batch_vector<T>   dU(size_U, 1, stU, bc);
     device_strided_batch_vector<int> dinfo(1, 1, 1, bc);
-    device_strided_batch_vector<T>   dWork(size_W, 1, size_W, bc);
+    device_strided_batch_vector<T>   dWork(size_W, 1, size_W, 1); // size_W accounts for bc
     if(size_S)
         CHECK_HIP_ERROR(dS.memcheck());
     if(size_V)
