@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -46,7 +46,6 @@ void syevdx_heevdx_checkBadArgs(const hipsolverHandle_t   handle,
                                 U                         dinfo,
                                 const int                 bc)
 {
-#if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // handle
     EXPECT_ROCBLAS_STATUS(hipsolver_syevdx_heevdx(API,
                                                   nullptr,
@@ -135,6 +134,7 @@ void syevdx_heevdx_checkBadArgs(const hipsolverHandle_t   handle,
                                                   bc),
                           HIPSOLVER_STATUS_INVALID_ENUM);
 
+#if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // pointers
     EXPECT_ROCBLAS_STATUS(hipsolver_syevdx_heevdx(API,
                                                   handle,
@@ -271,7 +271,7 @@ void testing_syevdx_heevdx_bad_arg()
         //                                    hNev.data(),
         //                                    dW.data(),
         //                                    &size_W);
-        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         // if(size_W)
         //     CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -323,7 +323,7 @@ void testing_syevdx_heevdx_bad_arg()
                                            hNev.data(),
                                            dW.data(),
                                            &size_W);
-        device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         if(size_W)
             CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -493,39 +493,45 @@ void syevdx_heevdx_getError(const hipsolverHandle_t   handle,
 
     // CPU lapack
     for(int b = 0; b < bc; ++b)
-        cblas_syevx_heevx<T>(evect,
-                             erange,
-                             uplo,
-                             n,
-                             hA[b],
-                             lda,
-                             vl,
-                             vu,
-                             il,
-                             iu,
-                             abstol,
-                             hNev[b],
-                             hW[b],
-                             Z.data(),
-                             lda,
-                             work.data(),
-                             size_work,
-                             rwork.data(),
-                             iwork.data(),
-                             ifail.data(),
-                             hinfo[b]);
+        cpu_syevx_heevx(evect,
+                        erange,
+                        uplo,
+                        n,
+                        hA[b],
+                        lda,
+                        vl,
+                        vu,
+                        il,
+                        iu,
+                        abstol,
+                        hNev[b],
+                        hW[b],
+                        Z.data(),
+                        lda,
+                        work.data(),
+                        size_work,
+                        rwork.data(),
+                        iwork.data(),
+                        ifail.data(),
+                        hinfo[b]);
 
     // Check info for non-convergence
     *max_err = 0;
     for(int b = 0; b < bc; ++b)
+    {
+        EXPECT_EQ(hinfo[b][0], hinfoRes[b][0]) << "where b = " << b;
         if(hinfo[b][0] != hinfoRes[b][0])
             *max_err += 1;
+    }
 
     // Check number of returned eigenvalues
     *max_err = 0;
     for(int b = 0; b < bc; ++b)
+    {
+        EXPECT_EQ(hNev[b][0], hNevRes[b][0]) << "where b = " << b;
         if(hNev[b][0] != hNevRes[b][0])
             *max_err += 1;
+    }
 
     // (We expect the used input matrices to always converge. Testing
     // implicitly the equivalent non-converged matrix is very complicated and it boils
@@ -558,16 +564,16 @@ void syevdx_heevdx_getError(const hipsolverHandle_t   handle,
                 for(int j = 0; j < hNev[b][0]; j++)
                 {
                     alpha = T(1) / hWRes[b][j];
-                    cblas_symv_hemv(uplo,
-                                    n,
-                                    alpha,
-                                    A.data() + b * lda * n,
-                                    lda,
-                                    hARes[b] + j * lda,
-                                    1,
-                                    beta,
-                                    hA[b] + j * lda,
-                                    1);
+                    cpu_symv_hemv(uplo,
+                                  n,
+                                  alpha,
+                                  A.data() + b * lda * n,
+                                  lda,
+                                  hARes[b] + j * lda,
+                                  1,
+                                  beta,
+                                  hA[b] + j * lda,
+                                  1);
                 }
 
                 // error is ||hA - hARes|| / ||hA||
@@ -638,27 +644,27 @@ void syevdx_heevdx_getPerfData(const hipsolverHandle_t   handle,
         // cpu-lapack performance (only if not in perf mode)
         *cpu_time_used = get_time_us_no_sync();
         for(int b = 0; b < bc; ++b)
-            cblas_syevx_heevx<T>(evect,
-                                 erange,
-                                 uplo,
-                                 n,
-                                 hA[b],
-                                 lda,
-                                 vl,
-                                 vu,
-                                 il,
-                                 iu,
-                                 abstol,
-                                 hNev[b],
-                                 hW[b],
-                                 Z.data(),
-                                 lda,
-                                 work.data(),
-                                 size_work,
-                                 rwork.data(),
-                                 iwork.data(),
-                                 ifail.data(),
-                                 hinfo[b]);
+            cpu_syevx_heevx(evect,
+                            erange,
+                            uplo,
+                            n,
+                            hA[b],
+                            lda,
+                            vl,
+                            vu,
+                            il,
+                            iu,
+                            abstol,
+                            hNev[b],
+                            hW[b],
+                            Z.data(),
+                            lda,
+                            work.data(),
+                            size_work,
+                            rwork.data(),
+                            iwork.data(),
+                            ifail.data(),
+                            hinfo[b]);
         *cpu_time_used = get_time_us_no_sync() - *cpu_time_used;
     }
 
@@ -856,7 +862,7 @@ void testing_syevdx_heevdx(Arguments& argus)
     // device
     device_strided_batch_vector<S>   dW(size_W, 1, stW, bc);
     device_strided_batch_vector<int> dinfo(1, 1, 1, bc);
-    device_strided_batch_vector<T>   dWork(size_Work, 1, size_Work, bc);
+    device_strided_batch_vector<T>   dWork(size_Work, 1, size_Work, 1); // size_W accounts for bc
     if(size_W)
         CHECK_HIP_ERROR(dW.memcheck());
     CHECK_HIP_ERROR(dinfo.memcheck());
@@ -1012,9 +1018,9 @@ void testing_syevdx_heevdx(Arguments& argus)
     }
 
     // validate results for rocsolver-test
-    // using n * machine_precision as tolerance
+    // using 3 * n * machine_precision as tolerance
     if(argus.unit_check)
-        ROCSOLVER_TEST_CHECK(T, max_error, n);
+        ROCSOLVER_TEST_CHECK(T, max_error, 3 * n);
 
     // output results for rocsolver-bench
     if(argus.timing)
