@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2021-2022 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -41,7 +41,6 @@ void syevj_heevj_checkBadArgs(const hipsolverHandle_t    handle,
                               const hipsolverSyevjInfo_t params,
                               const int                  bc)
 {
-#if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // handle
     EXPECT_ROCBLAS_STATUS(hipsolver_syevj_heevj(API,
                                                 STRIDED,
@@ -97,6 +96,7 @@ void syevj_heevj_checkBadArgs(const hipsolverHandle_t    handle,
                                                 bc),
                           HIPSOLVER_STATUS_INVALID_ENUM);
 
+#if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // pointers
     EXPECT_ROCBLAS_STATUS(hipsolver_syevj_heevj(API,
                                                 STRIDED,
@@ -181,7 +181,7 @@ void testing_syevj_heevj_bad_arg()
         // int size_W;
         // hipsolver_syevj_heevj_bufferSize(
         //     API, handle, evect, uplo, n, dA.data(), lda, dD.data(), &size_W, params, bc);
-        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         // if(size_W)
         //     CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -214,7 +214,7 @@ void testing_syevj_heevj_bad_arg()
         int size_W;
         hipsolver_syevj_heevj_bufferSize(
             API, STRIDED, handle, evect, uplo, n, dA.data(), lda, dD.data(), &size_W, params, bc);
-        device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         if(size_W)
             CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -375,38 +375,54 @@ void syevj_heevj_getError(const hipsolverHandle_t   handle,
 
     // CPU lapack
     for(int b = 0; b < bc; ++b)
-        cblas_syevd_heevd<T>(evect,
-                             uplo,
-                             n,
-                             hA[b],
-                             lda,
-                             hD[b],
-                             work.data(),
-                             ltwork,
-                             hE.data(),
-                             sizeE,
-                             iwork.data(),
-                             liwork,
-                             hinfo[b]);
+        cpu_syevd_heevd(evect,
+                        uplo,
+                        n,
+                        hA[b],
+                        lda,
+                        hD[b],
+                        work.data(),
+                        ltwork,
+                        hE.data(),
+                        sizeE,
+                        iwork.data(),
+                        liwork,
+                        hinfo[b]);
 
     // Check info for non-convergence
     *max_err = 0;
     for(int b = 0; b < bc; ++b)
+    {
+        EXPECT_EQ(hinfo[b][0], hinfoRes[b][0]) << "where b = " << b;
         if(hinfo[b][0] != hinfoRes[b][0])
             *max_err += 1;
+    }
 
     if(!STRIDED)
     {
         // Also check validity of residual
         for(rocblas_int b = 0; b < bc; ++b)
-            if(hResidualRes[b][0] < 0
-               || hResidualRes[b][0] > snorm('F', n, n, A.data() + b * lda * n, lda) * atol)
+        {
+            EXPECT_GE(hResidualRes[b][0], 0) << "where b = " << b;
+            if(hResidualRes[b][0] < 0)
                 *max_err += 1;
+            else
+            {
+                S threshold = snorm('F', n, n, A.data() + b * lda * n, lda) * atol;
+                EXPECT_LE(hResidualRes[b][0], threshold) << "where b = " << b;
+                if(hResidualRes[b][0] > threshold)
+                    *max_err += 1;
+            }
+        }
 
         // Also check validity of sweeps
         for(rocblas_int b = 0; b < bc; ++b)
+        {
+            EXPECT_GE(hSweepsRes[b][0], 0) << "where b = " << b;
+            EXPECT_LE(hSweepsRes[b][0], max_sweeps) << "where b = " << b;
             if(hSweepsRes[b][0] < 0 || hSweepsRes[b][0] > max_sweeps)
                 *max_err += 1;
+        }
     }
 
     double err = 0;
@@ -436,16 +452,16 @@ void syevj_heevj_getError(const hipsolverHandle_t   handle,
                 for(int j = 0; j < n; j++)
                 {
                     alpha = T(1) / hDres[b][j];
-                    cblas_symv_hemv(uplo,
-                                    n,
-                                    alpha,
-                                    A.data() + b * lda * n,
-                                    lda,
-                                    hAres[b] + j * lda,
-                                    1,
-                                    beta,
-                                    hA[b] + j * lda,
-                                    1);
+                    cpu_symv_hemv(uplo,
+                                  n,
+                                  alpha,
+                                  A.data() + b * lda * n,
+                                  lda,
+                                  hAres[b] + j * lda,
+                                  1,
+                                  beta,
+                                  hA[b] + j * lda,
+                                  1);
                 }
 
                 // error is ||hA - hARes|| / ||hA||
@@ -516,19 +532,19 @@ void syevj_heevj_getPerfData(const hipsolverHandle_t   handle,
         // cpu-lapack performance (only if not in perf mode)
         *cpu_time_used = get_time_us_no_sync();
         for(int b = 0; b < bc; ++b)
-            cblas_syevd_heevd<T>(evect,
-                                 uplo,
-                                 n,
-                                 hA[b],
-                                 lda,
-                                 hD[b],
-                                 work.data(),
-                                 ltwork,
-                                 hE.data(),
-                                 sizeE,
-                                 iwork.data(),
-                                 liwork,
-                                 hinfo[b]);
+            cpu_syevd_heevd(evect,
+                            uplo,
+                            n,
+                            hA[b],
+                            lda,
+                            hD[b],
+                            work.data(),
+                            ltwork,
+                            hE.data(),
+                            sizeE,
+                            iwork.data(),
+                            liwork,
+                            hinfo[b]);
         *cpu_time_used = get_time_us_no_sync() - *cpu_time_used;
     }
 
@@ -696,7 +712,7 @@ void testing_syevj_heevj(Arguments& argus)
     // device
     device_strided_batch_vector<S>   dD(size_D, 1, stD, bc);
     device_strided_batch_vector<int> dinfo(1, 1, 1, bc);
-    device_strided_batch_vector<T>   dWork(size_W, 1, size_W, bc);
+    device_strided_batch_vector<T>   dWork(size_W, 1, size_W, 1); // size_W accounts for bc
     if(size_D)
         CHECK_HIP_ERROR(dD.memcheck());
     CHECK_HIP_ERROR(dinfo.memcheck());
