@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -372,7 +372,7 @@ void testing_sygvdx_hegvdx_bad_arg()
         //                                    hNev.data(),
         //                                    dW.data(),
         //                                    &size_W);
-        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         // if(size_W)
         //     CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -437,7 +437,7 @@ void testing_sygvdx_hegvdx_bad_arg()
                                            hNev.data(),
                                            dW.data(),
                                            &size_W);
-        device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         if(size_W)
             CHECK_HIP_ERROR(dWork.memcheck());
 
@@ -532,69 +532,69 @@ void sygvdx_hegvdx_initData(const hipsolverHandle_t       handle,
 
             // form B = U' U
             T one = T(1);
-            cblas_trmm<T>(HIPSOLVER_SIDE_LEFT,
-                          HIPSOLVER_FILL_MODE_UPPER,
-                          HIPSOLVER_OP_C,
-                          'N',
-                          n,
-                          n,
-                          one,
-                          U[b],
-                          ldu,
-                          hB[b],
-                          ldb);
+            cpu_trmm(HIPSOLVER_SIDE_LEFT,
+                     HIPSOLVER_FILL_MODE_UPPER,
+                     HIPSOLVER_OP_C,
+                     'N',
+                     n,
+                     n,
+                     one,
+                     U[b],
+                     ldu,
+                     hB[b],
+                     ldb);
 
             if(itype == HIPSOLVER_EIG_TYPE_1)
             {
                 // form A = U' M U
-                cblas_trmm<T>(HIPSOLVER_SIDE_LEFT,
-                              HIPSOLVER_FILL_MODE_UPPER,
-                              HIPSOLVER_OP_C,
-                              'N',
-                              n,
-                              n,
-                              one,
-                              U[b],
-                              ldu,
-                              hA[b],
-                              lda);
-                cblas_trmm<T>(HIPSOLVER_SIDE_RIGHT,
-                              HIPSOLVER_FILL_MODE_UPPER,
-                              HIPSOLVER_OP_N,
-                              'N',
-                              n,
-                              n,
-                              one,
-                              U[b],
-                              ldu,
-                              hA[b],
-                              lda);
+                cpu_trmm(HIPSOLVER_SIDE_LEFT,
+                         HIPSOLVER_FILL_MODE_UPPER,
+                         HIPSOLVER_OP_C,
+                         'N',
+                         n,
+                         n,
+                         one,
+                         U[b],
+                         ldu,
+                         hA[b],
+                         lda);
+                cpu_trmm(HIPSOLVER_SIDE_RIGHT,
+                         HIPSOLVER_FILL_MODE_UPPER,
+                         HIPSOLVER_OP_N,
+                         'N',
+                         n,
+                         n,
+                         one,
+                         U[b],
+                         ldu,
+                         hA[b],
+                         lda);
             }
             else
             {
                 // form A = inv(U) M inv(U')
-                cblas_trsm<T>(HIPSOLVER_SIDE_LEFT,
-                              HIPSOLVER_FILL_MODE_UPPER,
-                              HIPSOLVER_OP_N,
-                              'N',
-                              n,
-                              n,
-                              one,
-                              U[b],
-                              ldu,
-                              hA[b],
-                              lda);
-                cblas_trsm<T>(HIPSOLVER_SIDE_RIGHT,
-                              HIPSOLVER_FILL_MODE_UPPER,
-                              HIPSOLVER_OP_C,
-                              'N',
-                              n,
-                              n,
-                              one,
-                              U[b],
-                              ldu,
-                              hA[b],
-                              lda);
+                cpu_trsm(HIPSOLVER_SIDE_LEFT,
+                         HIPSOLVER_FILL_MODE_UPPER,
+                         HIPSOLVER_OP_N,
+                         'N',
+                         n,
+                         n,
+                         one,
+                         U[b],
+                         ldu,
+                         hA[b],
+                         lda);
+                cpu_trsm(HIPSOLVER_SIDE_RIGHT,
+                         HIPSOLVER_FILL_MODE_UPPER,
+                         HIPSOLVER_OP_C,
+                         'N',
+                         n,
+                         n,
+                         one,
+                         U[b],
+                         ldu,
+                         hA[b],
+                         lda);
             }
 
             // store A and B for testing purposes
@@ -724,30 +724,30 @@ void sygvdx_hegvdx_getError(const hipsolverHandle_t   handle,
     S abstol = 2 * get_safemin<S>();
     for(int b = 0; b < bc; ++b)
     {
-        cblas_sygvx_hegvx<T>(itype,
-                             evect,
-                             erange,
-                             uplo,
-                             n,
-                             hA[b],
-                             lda,
-                             hB[b],
-                             ldb,
-                             vl,
-                             vu,
-                             il,
-                             iu,
-                             abstol,
-                             hNev[b],
-                             hW[b],
-                             Z.data(),
-                             n,
-                             work.data(),
-                             size_work,
-                             rwork.data(),
-                             iwork.data(),
-                             ifail.data(),
-                             hInfo[b]);
+        cpu_sygvx_hegvx(itype,
+                        evect,
+                        erange,
+                        uplo,
+                        n,
+                        hA[b],
+                        lda,
+                        hB[b],
+                        ldb,
+                        vl,
+                        vu,
+                        il,
+                        iu,
+                        abstol,
+                        hNev[b],
+                        hW[b],
+                        Z.data(),
+                        n,
+                        work.data(),
+                        size_work,
+                        rwork.data(),
+                        iwork.data(),
+                        ifail.data(),
+                        hInfo[b]);
     }
 
     // (We expect the used input matrices to always converge. Testing
@@ -758,14 +758,20 @@ void sygvdx_hegvdx_getError(const hipsolverHandle_t   handle,
     // check info for non-convergence and/or positive-definiteness
     *max_err = 0;
     for(int b = 0; b < bc; ++b)
+    {
+        EXPECT_EQ(hInfo[b][0], hInfoRes[b][0]) << "where b = " << b;
         if(hInfo[b][0] != hInfoRes[b][0])
             *max_err += 1;
+    }
 
     // Check number of returned eigenvalues
     *max_err = 0;
     for(int b = 0; b < bc; ++b)
+    {
+        EXPECT_EQ(hNev[b][0], hNevRes[b][0]) << "where b = " << b;
         if(hNev[b][0] != hNevRes[b][0])
             *max_err += 1;
+    }
 
     double err;
 
@@ -794,18 +800,18 @@ void sygvdx_hegvdx_getError(const hipsolverHandle_t   handle,
 
                 // hARes contains eigenvectors x
                 // compute B*x (or A*x) and store in hB
-                cblas_symm_hemm<T>(HIPSOLVER_SIDE_LEFT,
-                                   uplo,
-                                   n,
-                                   hNev[b][0],
-                                   alpha,
-                                   B[b],
-                                   ldb,
-                                   hARes[b],
-                                   lda,
-                                   beta,
-                                   hB[b],
-                                   ldb);
+                cpu_symm_hemm(HIPSOLVER_SIDE_LEFT,
+                              uplo,
+                              n,
+                              hNev[b][0],
+                              alpha,
+                              B[b],
+                              ldb,
+                              hARes[b],
+                              lda,
+                              beta,
+                              hB[b],
+                              ldb);
 
                 if(itype == HIPSOLVER_EIG_TYPE_1)
                 {
@@ -815,16 +821,16 @@ void sygvdx_hegvdx_getError(const hipsolverHandle_t   handle,
                     for(int j = 0; j < hNev[b][0]; j++)
                     {
                         alpha = T(1) / hWRes[b][j];
-                        cblas_symv_hemv(uplo,
-                                        n,
-                                        alpha,
-                                        A[b],
-                                        lda,
-                                        hARes[b] + j * lda,
-                                        1,
-                                        beta,
-                                        hA[b] + j * lda,
-                                        1);
+                        cpu_symv_hemv(uplo,
+                                      n,
+                                      alpha,
+                                      A[b],
+                                      lda,
+                                      hARes[b] + j * lda,
+                                      1,
+                                      beta,
+                                      hA[b] + j * lda,
+                                      1);
                     }
 
                     // move B*x into hARes
@@ -840,16 +846,16 @@ void sygvdx_hegvdx_getError(const hipsolverHandle_t   handle,
                     for(int j = 0; j < hNev[b][0]; j++)
                     {
                         alpha = T(1) / hWRes[b][j];
-                        cblas_symv_hemv(uplo,
-                                        n,
-                                        alpha,
-                                        A[b],
-                                        lda,
-                                        hB[b] + j * ldb,
-                                        1,
-                                        beta,
-                                        hA[b] + j * lda,
-                                        1);
+                        cpu_symv_hemv(uplo,
+                                      n,
+                                      alpha,
+                                      A[b],
+                                      lda,
+                                      hB[b] + j * ldb,
+                                      1,
+                                      beta,
+                                      hA[b] + j * lda,
+                                      1);
                     }
                 }
 
@@ -929,30 +935,30 @@ void sygvdx_hegvdx_getPerfData(const hipsolverHandle_t   handle,
         *cpu_time_used = get_time_us_no_sync();
         for(int b = 0; b < bc; ++b)
         {
-            cblas_sygvx_hegvx<T>(itype,
-                                 evect,
-                                 erange,
-                                 uplo,
-                                 n,
-                                 hA[b],
-                                 lda,
-                                 hB[b],
-                                 ldb,
-                                 vl,
-                                 vu,
-                                 il,
-                                 iu,
-                                 abstol,
-                                 hNev[b],
-                                 hW[b],
-                                 Z.data(),
-                                 n,
-                                 work.data(),
-                                 size_work,
-                                 rwork.data(),
-                                 iwork.data(),
-                                 ifail.data(),
-                                 hInfo[b]);
+            cpu_sygvx_hegvx(itype,
+                            evect,
+                            erange,
+                            uplo,
+                            n,
+                            hA[b],
+                            lda,
+                            hB[b],
+                            ldb,
+                            vl,
+                            vu,
+                            il,
+                            iu,
+                            abstol,
+                            hNev[b],
+                            hW[b],
+                            Z.data(),
+                            n,
+                            work.data(),
+                            size_work,
+                            rwork.data(),
+                            iwork.data(),
+                            ifail.data(),
+                            hInfo[b]);
         }
         *cpu_time_used = get_time_us_no_sync() - *cpu_time_used;
     }
@@ -1182,7 +1188,7 @@ void testing_sygvdx_hegvdx(Arguments& argus)
     // device
     device_strided_batch_vector<S>   dW(size_W, 1, stW, bc);
     device_strided_batch_vector<int> dInfo(1, 1, 1, bc);
-    device_strided_batch_vector<T>   dWork(size_Work, 1, size_Work, bc);
+    device_strided_batch_vector<T>   dWork(size_Work, 1, size_Work, 1); // size_W accounts for bc
     if(size_W)
         CHECK_HIP_ERROR(dW.memcheck());
     CHECK_HIP_ERROR(dInfo.memcheck());
@@ -1358,9 +1364,9 @@ void testing_sygvdx_hegvdx(Arguments& argus)
     }
 
     // validate results for rocsolver-test
-    // using 2 * n * machine_precision as tolerance
+    // using 3 * n * machine_precision as tolerance
     if(argus.unit_check)
-        ROCSOLVER_TEST_CHECK(T, max_error, 2 * n);
+        ROCSOLVER_TEST_CHECK(T, max_error, 3 * n);
 
     // output results for rocsolver-bench
     if(argus.timing)
