@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -25,7 +25,7 @@
 
 #include "clientcommon.hpp"
 
-template <bool FORTRAN, typename S, typename T, typename U, typename V>
+template <testAPI_t API, typename S, typename T, typename U, typename V>
 void sytrd_hetrd_checkBadArgs(const hipsolverHandle_t   handle,
                               const hipsolverFillMode_t uplo,
                               const int                 n,
@@ -44,7 +44,7 @@ void sytrd_hetrd_checkBadArgs(const hipsolverHandle_t   handle,
                               const int                 bc)
 {
     // handle
-    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(FORTRAN,
+    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(API,
                                                 nullptr,
                                                 uplo,
                                                 n,
@@ -64,7 +64,7 @@ void sytrd_hetrd_checkBadArgs(const hipsolverHandle_t   handle,
                           HIPSOLVER_STATUS_NOT_INITIALIZED);
 
     // values
-    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(FORTRAN,
+    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(API,
                                                 handle,
                                                 hipsolverFillMode_t(-1),
                                                 n,
@@ -85,7 +85,7 @@ void sytrd_hetrd_checkBadArgs(const hipsolverHandle_t   handle,
 
 #if defined(__HIP_PLATFORM_HCC__) || defined(__HIP_PLATFORM_AMD__)
     // pointers
-    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(FORTRAN,
+    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(API,
                                                 handle,
                                                 uplo,
                                                 n,
@@ -103,7 +103,7 @@ void sytrd_hetrd_checkBadArgs(const hipsolverHandle_t   handle,
                                                 dInfo,
                                                 bc),
                           HIPSOLVER_STATUS_INVALID_VALUE);
-    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(FORTRAN,
+    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(API,
                                                 handle,
                                                 uplo,
                                                 n,
@@ -121,7 +121,7 @@ void sytrd_hetrd_checkBadArgs(const hipsolverHandle_t   handle,
                                                 dInfo,
                                                 bc),
                           HIPSOLVER_STATUS_INVALID_VALUE);
-    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(FORTRAN,
+    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(API,
                                                 handle,
                                                 uplo,
                                                 n,
@@ -139,7 +139,7 @@ void sytrd_hetrd_checkBadArgs(const hipsolverHandle_t   handle,
                                                 dInfo,
                                                 bc),
                           HIPSOLVER_STATUS_INVALID_VALUE);
-    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(FORTRAN,
+    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(API,
                                                 handle,
                                                 uplo,
                                                 n,
@@ -157,7 +157,7 @@ void sytrd_hetrd_checkBadArgs(const hipsolverHandle_t   handle,
                                                 dInfo,
                                                 bc),
                           HIPSOLVER_STATUS_INVALID_VALUE);
-    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(FORTRAN,
+    EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(API,
                                                 handle,
                                                 uplo,
                                                 n,
@@ -178,7 +178,7 @@ void sytrd_hetrd_checkBadArgs(const hipsolverHandle_t   handle,
 #endif
 }
 
-template <bool FORTRAN, bool BATCHED, bool STRIDED, typename T>
+template <testAPI_t API, bool BATCHED, bool STRIDED, typename T>
 void testing_sytrd_hetrd_bad_arg()
 {
     using S = decltype(std::real(T{}));
@@ -186,8 +186,8 @@ void testing_sytrd_hetrd_bad_arg()
     // safe arguments
     hipsolver_local_handle handle;
     hipsolverFillMode_t    uplo = HIPSOLVER_FILL_MODE_UPPER;
-    int                    n    = 1;
-    int                    lda  = 1;
+    int                    n    = 2;
+    int                    lda  = 2;
     int                    stA  = 1;
     int                    stD  = 1;
     int                    stE  = 1;
@@ -210,13 +210,13 @@ void testing_sytrd_hetrd_bad_arg()
 
         // int size_W;
         // hipsolver_sytrd_hetrd_bufferSize(
-        //     FORTRAN, handle, uplo, n, dA.data(), lda, dD.data(), dE.data(), dTau.data(), &size_W);
-        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+        //     API, handle, uplo, n, dA.data(), lda, dD.data(), dE.data(), dTau.data(), &size_W);
+        // device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         // if(size_W)
         //     CHECK_HIP_ERROR(dWork.memcheck());
 
         // // check bad arguments
-        // sytrd_hetrd_checkBadArgs<FORTRAN>(handle,
+        // sytrd_hetrd_checkBadArgs<API>(handle,
         //                                   uplo,
         //                                   n,
         //                                   dA.data(),
@@ -249,28 +249,28 @@ void testing_sytrd_hetrd_bad_arg()
 
         int size_W;
         hipsolver_sytrd_hetrd_bufferSize(
-            FORTRAN, handle, uplo, n, dA.data(), lda, dD.data(), dE.data(), dTau.data(), &size_W);
-        device_strided_batch_vector<T> dWork(size_W, 1, size_W, bc);
+            API, handle, uplo, n, dA.data(), lda, dD.data(), dE.data(), dTau.data(), &size_W);
+        device_strided_batch_vector<T> dWork(size_W, 1, size_W, 1);
         if(size_W)
             CHECK_HIP_ERROR(dWork.memcheck());
 
         // check bad arguments
-        sytrd_hetrd_checkBadArgs<FORTRAN>(handle,
-                                          uplo,
-                                          n,
-                                          dA.data(),
-                                          lda,
-                                          stA,
-                                          dD.data(),
-                                          stD,
-                                          dE.data(),
-                                          stE,
-                                          dTau.data(),
-                                          stP,
-                                          dWork.data(),
-                                          size_W,
-                                          dInfo.data(),
-                                          bc);
+        sytrd_hetrd_checkBadArgs<API>(handle,
+                                      uplo,
+                                      n,
+                                      dA.data(),
+                                      lda,
+                                      stA,
+                                      dD.data(),
+                                      stD,
+                                      dE.data(),
+                                      stE,
+                                      dTau.data(),
+                                      stP,
+                                      dWork.data(),
+                                      size_W,
+                                      dInfo.data(),
+                                      bc);
     }
 }
 
@@ -348,7 +348,7 @@ void sytrd_hetrd_initData(
     }
 }
 
-template <bool FORTRAN,
+template <testAPI_t API,
           typename T,
           typename Sd,
           typename Td,
@@ -393,7 +393,7 @@ void sytrd_hetrd_getError(const hipsolverHandle_t   handle,
 
     // execute computations
     // GPU lapack
-    CHECK_ROCBLAS_ERROR(hipsolver_sytrd_hetrd(FORTRAN,
+    CHECK_ROCBLAS_ERROR(hipsolver_sytrd_hetrd(API,
                                               handle,
                                               uplo,
                                               n,
@@ -445,26 +445,26 @@ void sytrd_hetrd_getError(const hipsolverHandle_t   handle,
                 v[0] = 1;
 
                 // apply householder reflector
-                cblas_larf(HIPSOLVER_SIDE_LEFT,
-                           n - 1 - j,
-                           n - j,
-                           v.data(),
-                           1,
-                           t + j,
-                           a + (j + 1) + j * lda,
-                           lda,
-                           hW.data());
+                cpu_larf(HIPSOLVER_SIDE_LEFT,
+                         n - 1 - j,
+                         n - j,
+                         v.data(),
+                         1,
+                         t + j,
+                         a + (j + 1) + j * lda,
+                         lda,
+                         hW.data());
                 if(COMPLEX)
-                    cblas_lacgv(1, t + j, 1);
-                cblas_larf(HIPSOLVER_SIDE_RIGHT,
-                           n - j,
-                           n - 1 - j,
-                           v.data(),
-                           1,
-                           t + j,
-                           a + j + (j + 1) * lda,
-                           lda,
-                           hW.data());
+                    cpu_lacgv(1, t + j, 1);
+                cpu_larf(HIPSOLVER_SIDE_RIGHT,
+                         n - j,
+                         n - 1 - j,
+                         v.data(),
+                         1,
+                         t + j,
+                         a + j + (j + 1) * lda,
+                         lda,
+                         hW.data());
             }
         }
 
@@ -490,12 +490,10 @@ void sytrd_hetrd_getError(const hipsolverHandle_t   handle,
                     a[i + j * lda] = 0;
 
                 // apply householder reflector
-                cblas_larf(
-                    HIPSOLVER_SIDE_LEFT, j, j + 1, v.data(), 1, t + j - 1, a, lda, hW.data());
+                cpu_larf(HIPSOLVER_SIDE_LEFT, j, j + 1, v.data(), 1, t + j - 1, a, lda, hW.data());
                 if(COMPLEX)
-                    cblas_lacgv(1, t + j - 1, 1);
-                cblas_larf(
-                    HIPSOLVER_SIDE_RIGHT, j + 1, j, v.data(), 1, t + j - 1, a, lda, hW.data());
+                    cpu_lacgv(1, t + j - 1, 1);
+                cpu_larf(HIPSOLVER_SIDE_RIGHT, j + 1, j, v.data(), 1, t + j - 1, a, lda, hW.data());
             }
         }
     }
@@ -514,12 +512,15 @@ void sytrd_hetrd_getError(const hipsolverHandle_t   handle,
     // check info
     err = 0;
     for(int b = 0; b < bc; ++b)
+    {
+        EXPECT_EQ(hInfoRes[b][0], 0) << "where b = " << b;
         if(hInfoRes[b][0] != 0)
             err++;
+    }
     *max_err += err;
 }
 
-template <bool FORTRAN,
+template <testAPI_t API,
           typename T,
           typename Sd,
           typename Td,
@@ -566,7 +567,7 @@ void sytrd_hetrd_getPerfData(const hipsolverHandle_t   handle,
         // cpu-lapack performance (only if not in perf mode)
         *cpu_time_used = get_time_us_no_sync();
         for(int b = 0; b < bc; ++b)
-            cblas_sytrd_hetrd<T, S>(uplo, n, hA[b], lda, hD[b], hE[b], hTau[b], hW.data(), 32 * n);
+            cpu_sytrd_hetrd(uplo, n, hA[b], lda, hD[b], hE[b], hTau[b], hW.data(), 32 * n);
         *cpu_time_used = get_time_us_no_sync() - *cpu_time_used;
     }
 
@@ -577,7 +578,7 @@ void sytrd_hetrd_getPerfData(const hipsolverHandle_t   handle,
     {
         sytrd_hetrd_initData<false, true, T>(handle, n, dA, lda, bc, hA);
 
-        CHECK_ROCBLAS_ERROR(hipsolver_sytrd_hetrd(FORTRAN,
+        CHECK_ROCBLAS_ERROR(hipsolver_sytrd_hetrd(API,
                                                   handle,
                                                   uplo,
                                                   n,
@@ -606,7 +607,7 @@ void sytrd_hetrd_getPerfData(const hipsolverHandle_t   handle,
         sytrd_hetrd_initData<false, true, T>(handle, n, dA, lda, bc, hA);
 
         start = get_time_us_sync(stream);
-        hipsolver_sytrd_hetrd(FORTRAN,
+        hipsolver_sytrd_hetrd(API,
                               handle,
                               uplo,
                               n,
@@ -628,7 +629,7 @@ void sytrd_hetrd_getPerfData(const hipsolverHandle_t   handle,
     *gpu_time_used /= hot_calls;
 }
 
-template <bool FORTRAN, bool BATCHED, bool STRIDED, typename T>
+template <testAPI_t API, bool BATCHED, bool STRIDED, typename T>
 void testing_sytrd_hetrd(Arguments& argus)
 {
     using S = decltype(std::real(T{}));
@@ -654,7 +655,7 @@ void testing_sytrd_hetrd(Arguments& argus)
     {
         if(BATCHED)
         {
-            // EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(FORTRAN,
+            // EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(API,
             //                                             handle,
             //                                             uplo,
             //                                             n,
@@ -675,7 +676,7 @@ void testing_sytrd_hetrd(Arguments& argus)
         }
         else
         {
-            EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(FORTRAN,
+            EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(API,
                                                         handle,
                                                         uplo,
                                                         n,
@@ -716,7 +717,7 @@ void testing_sytrd_hetrd(Arguments& argus)
     {
         if(BATCHED)
         {
-            // EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(FORTRAN,
+            // EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(API,
             //                                             handle,
             //                                             uplo,
             //                                             n,
@@ -737,7 +738,7 @@ void testing_sytrd_hetrd(Arguments& argus)
         }
         else
         {
-            EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(FORTRAN,
+            EXPECT_ROCBLAS_STATUS(hipsolver_sytrd_hetrd(API,
                                                         handle,
                                                         uplo,
                                                         n,
@@ -766,7 +767,7 @@ void testing_sytrd_hetrd(Arguments& argus)
     // memory size query is necessary
     int size_W;
     hipsolver_sytrd_hetrd_bufferSize(
-        FORTRAN, handle, uplo, n, (T*)nullptr, lda, (S*)nullptr, (S*)nullptr, (T*)nullptr, &size_W);
+        API, handle, uplo, n, (T*)nullptr, lda, (S*)nullptr, (S*)nullptr, (T*)nullptr, &size_W);
 
     if(argus.mem_query)
     {
@@ -786,7 +787,7 @@ void testing_sytrd_hetrd(Arguments& argus)
     device_strided_batch_vector<S>   dE(size_E, 1, stE, bc);
     device_strided_batch_vector<T>   dTau(size_tau, 1, stP, bc);
     device_strided_batch_vector<int> dInfo(1, 1, 1, bc);
-    device_strided_batch_vector<T>   dWork(size_W, 1, size_W, bc);
+    device_strided_batch_vector<T>   dWork(size_W, 1, size_W, 1); // size_W accounts for bc
     if(size_D)
         CHECK_HIP_ERROR(dD.memcheck());
     if(size_E)
@@ -808,7 +809,7 @@ void testing_sytrd_hetrd(Arguments& argus)
 
         // // check computations
         // if(argus.unit_check || argus.norm_check)
-        //     sytrd_hetrd_getError<FORTRAN, T>(handle,
+        //     sytrd_hetrd_getError<API, T>(handle,
         //                                      uplo,
         //                                      n,
         //                                      dA,
@@ -835,7 +836,7 @@ void testing_sytrd_hetrd(Arguments& argus)
 
         // // collect performance data
         // if(argus.timing)
-        //     sytrd_hetrd_getPerfData<FORTRAN, T>(handle,
+        //     sytrd_hetrd_getPerfData<API, T>(handle,
         //                                         uplo,
         //                                         n,
         //                                         dA,
@@ -873,58 +874,58 @@ void testing_sytrd_hetrd(Arguments& argus)
 
         // check computations
         if(argus.unit_check || argus.norm_check)
-            sytrd_hetrd_getError<FORTRAN, T>(handle,
-                                             uplo,
-                                             n,
-                                             dA,
-                                             lda,
-                                             stA,
-                                             dD,
-                                             stD,
-                                             dE,
-                                             stE,
-                                             dTau,
-                                             stP,
-                                             dWork,
-                                             size_W,
-                                             dInfo,
-                                             bc,
-                                             hA,
-                                             hARes,
-                                             hD,
-                                             hE,
-                                             hTau,
-                                             hInfo,
-                                             hInfoRes,
-                                             &max_error);
+            sytrd_hetrd_getError<API, T>(handle,
+                                         uplo,
+                                         n,
+                                         dA,
+                                         lda,
+                                         stA,
+                                         dD,
+                                         stD,
+                                         dE,
+                                         stE,
+                                         dTau,
+                                         stP,
+                                         dWork,
+                                         size_W,
+                                         dInfo,
+                                         bc,
+                                         hA,
+                                         hARes,
+                                         hD,
+                                         hE,
+                                         hTau,
+                                         hInfo,
+                                         hInfoRes,
+                                         &max_error);
 
         // collect performance data
         if(argus.timing)
-            sytrd_hetrd_getPerfData<FORTRAN, T>(handle,
-                                                uplo,
-                                                n,
-                                                dA,
-                                                lda,
-                                                stA,
-                                                dD,
-                                                stD,
-                                                dE,
-                                                stE,
-                                                dTau,
-                                                stP,
-                                                dWork,
-                                                size_W,
-                                                dInfo,
-                                                bc,
-                                                hA,
-                                                hD,
-                                                hE,
-                                                hTau,
-                                                hInfo,
-                                                &gpu_time_used,
-                                                &cpu_time_used,
-                                                hot_calls,
-                                                argus.perf);
+            sytrd_hetrd_getPerfData<API, T>(handle,
+                                            uplo,
+                                            n,
+                                            dA,
+                                            lda,
+                                            stA,
+                                            dD,
+                                            stD,
+                                            dE,
+                                            stE,
+                                            dTau,
+                                            stP,
+                                            dWork,
+                                            size_W,
+                                            dInfo,
+                                            bc,
+                                            hA,
+                                            hD,
+                                            hE,
+                                            hTau,
+                                            hInfo,
+                                            &gpu_time_used,
+                                            &cpu_time_used,
+                                            hot_calls,
+                                            argus.perf);
     }
 
     // validate results for rocsolver-test
