@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,8 +28,8 @@
 #ifdef __cplusplus
 #include "complex.hpp"
 #include "hipsolver_datatype2string.hpp"
+#include <cassert>
 #include <cmath>
-#include <immintrin.h>
 #include <random>
 #include <type_traits>
 #include <vector>
@@ -90,6 +90,24 @@ inline void hipsolver_expect_status(hipsolverStatus_t status, hipsolverStatus_t
 #define EXPECT_ROCBLAS_STATUS(status, expected) hipsolver_expect_status(status, expected)
 #define CHECK_ROCBLAS_ERROR(status) hipsolver_expect_status(status, HIPSOLVER_STATUS_SUCCESS)
 
+// The info provided to EXPECT macros is used in hipsolver-test, but
+// in hipsolver-bench, the information is just discarded.
+struct hipsolver_info_discarder
+{
+    template <typename T>
+    hipsolver_info_discarder& operator<<(T&&)
+    {
+        return *this;
+    }
+};
+
+#define EXPECT_EQ(v1, v2) hipsolver_info_discarder()
+#define EXPECT_NE(v1, v2) hipsolver_info_discarder()
+#define EXPECT_LT(v1, v2) hipsolver_info_discarder()
+#define EXPECT_LE(v1, v2) hipsolver_info_discarder()
+#define EXPECT_GT(v1, v2) hipsolver_info_discarder()
+#define EXPECT_GE(v1, v2) hipsolver_info_discarder()
+
 #endif
 
 #ifdef __cplusplus
@@ -104,7 +122,8 @@ class hipsolver_local_handle
 public:
     hipsolver_local_handle()
     {
-        hipsolverCreate(&m_handle);
+        if(hipsolverCreate(&m_handle) != HIPSOLVER_STATUS_SUCCESS)
+            throw std::runtime_error("ERROR: Could not create hipsolverHandle_t");
     }
     ~hipsolver_local_handle()
     {
@@ -112,8 +131,11 @@ public:
     }
 
     hipsolver_local_handle(const hipsolver_local_handle&) = delete;
-    hipsolver_local_handle(hipsolver_local_handle&&)      = delete;
+
+    hipsolver_local_handle(hipsolver_local_handle&&) = delete;
+
     hipsolver_local_handle& operator=(const hipsolver_local_handle&) = delete;
+
     hipsolver_local_handle& operator=(hipsolver_local_handle&&) = delete;
 
     // Allow hipsolver_local_handle to be used anywhere hipsolverHandle_t is expected
@@ -127,6 +149,74 @@ public:
     }
 };
 
+class hipsolverRf_local_handle
+{
+    hipsolverRfHandle_t m_handle;
+
+public:
+    hipsolverRf_local_handle()
+    {
+        if(hipsolverRfCreate(&m_handle) != HIPSOLVER_STATUS_SUCCESS)
+            throw std::runtime_error("ERROR: Could not create hipsolverRfHandle_t");
+    }
+    ~hipsolverRf_local_handle()
+    {
+        hipsolverRfDestroy(m_handle);
+    }
+
+    hipsolverRf_local_handle(const hipsolverRf_local_handle&) = delete;
+
+    hipsolverRf_local_handle(hipsolverRf_local_handle&&) = delete;
+
+    hipsolverRf_local_handle& operator=(const hipsolverRf_local_handle&) = delete;
+
+    hipsolverRf_local_handle& operator=(hipsolverRf_local_handle&&) = delete;
+
+    // Allow hipsolverRf_local_handle to be used anywhere hipsolverRfHandle_t is expected
+    operator hipsolverRfHandle_t&()
+    {
+        return m_handle;
+    }
+    operator const hipsolverRfHandle_t&() const
+    {
+        return m_handle;
+    }
+};
+
+class hipsolverSp_local_handle
+{
+    hipsolverSpHandle_t m_handle;
+
+public:
+    hipsolverSp_local_handle()
+    {
+        if(hipsolverSpCreate(&m_handle) != HIPSOLVER_STATUS_SUCCESS)
+            throw std::runtime_error("ERROR: Could not create hipsolverSpHandle_t");
+    }
+    ~hipsolverSp_local_handle()
+    {
+        hipsolverSpDestroy(m_handle);
+    }
+
+    hipsolverSp_local_handle(const hipsolverSp_local_handle&) = delete;
+
+    hipsolverSp_local_handle(hipsolverSp_local_handle&&) = delete;
+
+    hipsolverSp_local_handle& operator=(const hipsolverSp_local_handle&) = delete;
+
+    hipsolverSp_local_handle& operator=(hipsolverSp_local_handle&&) = delete;
+
+    // Allow hipsolverSp_local_handle to be used anywhere hipsolverSpHandle_t is expected
+    operator hipsolverSpHandle_t&()
+    {
+        return m_handle;
+    }
+    operator const hipsolverSpHandle_t&() const
+    {
+        return m_handle;
+    }
+};
+
 /* ============================================================================================
  */
 /*! \brief  local gesvdj params which is automatically created and destroyed  */
@@ -137,7 +227,8 @@ class hipsolver_local_gesvdj_info
 public:
     hipsolver_local_gesvdj_info()
     {
-        hipsolverDnCreateGesvdjInfo(&m_info);
+        if(hipsolverDnCreateGesvdjInfo(&m_info) != HIPSOLVER_STATUS_SUCCESS)
+            throw std::runtime_error("ERROR: Could not create hipsolverGesvdjInfo_t");
     }
     ~hipsolver_local_gesvdj_info()
     {
@@ -145,8 +236,11 @@ public:
     }
 
     hipsolver_local_gesvdj_info(const hipsolver_local_gesvdj_info&) = delete;
-    hipsolver_local_gesvdj_info(hipsolver_local_gesvdj_info&&)      = delete;
+
+    hipsolver_local_gesvdj_info(hipsolver_local_gesvdj_info&&) = delete;
+
     hipsolver_local_gesvdj_info& operator=(const hipsolver_local_gesvdj_info&) = delete;
+
     hipsolver_local_gesvdj_info& operator=(hipsolver_local_gesvdj_info&&) = delete;
 
     // Allow hipsolver_local_gesvdj_info to be used anywhere hipsolverGesvdjInfo_t is expected
@@ -170,7 +264,8 @@ class hipsolver_local_syevj_info
 public:
     hipsolver_local_syevj_info()
     {
-        hipsolverDnCreateSyevjInfo(&m_info);
+        if(hipsolverDnCreateSyevjInfo(&m_info) != HIPSOLVER_STATUS_SUCCESS)
+            throw std::runtime_error("ERROR: Could not create hipsolverSyevjInfo_t");
     }
     ~hipsolver_local_syevj_info()
     {
@@ -178,8 +273,11 @@ public:
     }
 
     hipsolver_local_syevj_info(const hipsolver_local_syevj_info&) = delete;
-    hipsolver_local_syevj_info(hipsolver_local_syevj_info&&)      = delete;
+
+    hipsolver_local_syevj_info(hipsolver_local_syevj_info&&) = delete;
+
     hipsolver_local_syevj_info& operator=(const hipsolver_local_syevj_info&) = delete;
+
     hipsolver_local_syevj_info& operator=(hipsolver_local_syevj_info&&) = delete;
 
     // Allow hipsolver_local_syevj_info to be used anywhere hipsolverSyevjInfo_t is expected
@@ -193,6 +291,43 @@ public:
     }
 };
 
+/* ============================================================================================
+ */
+/*! \brief  local params which is automatically created and destroyed  */
+class hipsolver_local_params
+{
+    hipsolverDnParams_t m_info;
+
+public:
+    hipsolver_local_params()
+    {
+        if(hipsolverDnCreateParams(&m_info) != HIPSOLVER_STATUS_SUCCESS)
+            throw std::runtime_error("ERROR: Could not create hipsolverDnParams_t");
+    }
+    ~hipsolver_local_params()
+    {
+        hipsolverDnDestroyParams(m_info);
+    }
+
+    hipsolver_local_params(const hipsolver_local_params&) = delete;
+
+    hipsolver_local_params(hipsolver_local_params&&) = delete;
+
+    hipsolver_local_params& operator=(const hipsolver_local_params&) = delete;
+
+    hipsolver_local_params& operator=(hipsolver_local_params&&) = delete;
+
+    // Allow hipsolver_local_params to be used anywhere hipsolverDnParams_t is expected
+    operator hipsolverDnParams_t&()
+    {
+        return m_info;
+    }
+    operator const hipsolverDnParams_t&() const
+    {
+        return m_info;
+    }
+};
+
 /* ============================================================================================
  */
 
@@ -329,8 +464,7 @@ public:
 template <typename T>
 T random_generator()
 {
-    // return rand()/( (T)RAND_MAX + 1);
-    return T(rand() % 10 + 1);
+    return std::uniform_int_distribution<int>(1, 10)(hipsolver_rng);
 };
 
 // for hipsolverComplex, generate 2 floats
@@ -338,7 +472,8 @@ T random_generator()
 template <>
 inline hipsolverComplex random_generator<hipsolverComplex>()
 {
-    return hipsolverComplex(rand() % 10 + 1, rand() % 10 + 1);
+    return hipsolverComplex(float(std::uniform_int_distribution<int>(1, 10)(hipsolver_rng)),
+                            float(std::uniform_int_distribution<int>(1, 10)(hipsolver_rng)));
 }
 
 // for hipsolverDoubleComplex, generate 2 doubles
@@ -346,15 +481,15 @@ inline hipsolverComplex random_generator<hipsolverComplex>()
 template <>
 inline hipsolverDoubleComplex random_generator<hipsolverDoubleComplex>()
 {
-    return hipsolverDoubleComplex(rand() % 10 + 1, rand() % 10 + 1);
+    return hipsolverDoubleComplex(double(std::uniform_int_distribution<int>(1, 10)(hipsolver_rng)),
+                                  double(std::uniform_int_distribution<int>(1, 10)(hipsolver_rng)));
 }
 
 /*! \brief  generate a random number in range [-1,-2,-3,-4,-5,-6,-7,-8,-9,-10] */
 template <typename T>
 inline T random_generator_negative()
 {
-    // return rand()/( (T)RAND_MAX + 1);
-    return -T(rand() % 10 + 1);
+    return std::uniform_int_distribution<int>(-10, -1)(hipsolver_rng);
 };
 
 // for complex, generate two values, convert both to negative
@@ -364,13 +499,16 @@ inline T random_generator_negative()
 template <>
 inline hipsolverComplex random_generator_negative<hipsolverComplex>()
 {
-    return {float(-(rand() % 10 + 1)), float(-(rand() % 10 + 1))};
+    return hipsolverComplex(float(std::uniform_int_distribution<int>(-10, -1)(hipsolver_rng)),
+                            float(std::uniform_int_distribution<int>(-10, -1)(hipsolver_rng)));
 }
 
 template <>
 inline hipsolverDoubleComplex random_generator_negative<hipsolverDoubleComplex>()
 {
-    return {double(-(rand() % 10 + 1)), double(-(rand() % 10 + 1))};
+    return hipsolverDoubleComplex(
+        double(std::uniform_int_distribution<int>(-10, -1)(hipsolver_rng)),
+        double(std::uniform_int_distribution<int>(-10, -1)(hipsolver_rng)));
 }
 
 /* ============================================================================================ */
@@ -406,6 +544,27 @@ int type2int(T val);
 
 /* ============================================================================================ */
 /*! \brief  Debugging purpose, print out CPU and GPU result matrix, not valid in complex number  */
+template <typename T, std::enable_if_t<!is_complex<T>, int> = 0>
+void print_matrix(T* result, int m, int n, int lda)
+{
+    for(int i = 0; i < m; i++)
+        for(int j = 0; j < n; j++)
+            printf("matrix  col %d, row %d, result=%.8g\n", j, i, double(result[i + j * lda]));
+}
+
+/*! \brief  Debugging purpose, print out CPU and GPU result matrix, valid for complex number  */
+template <typename T, std::enable_if_t<+is_complex<T>, int> = 0>
+void print_matrix(T* result, int m, int n, int lda)
+{
+    for(int i = 0; i < m; i++)
+        for(int j = 0; j < n; j++)
+            printf("matrix  col %d, row %d, result=(%.8g,%.8g)\n",
+                   j,
+                   i,
+                   double(result[i + j * lda].real()),
+                   double(result[i + j * lda].imag()));
+}
+
 template <typename T, std::enable_if_t<!is_complex<T>, int> = 0>
 void print_matrix(T* CPU_result, T* GPU_result, int m, int n, int lda)
 {
@@ -433,6 +592,193 @@ void print_matrix(T* CPU_result, T* GPU_result, int m, int n, int lda)
                    double(GPU_result[i + j * lda].imag()));
 }
 
+/*! \brief  Debugging purpose, print out sparse matrix, not valid in complex number  */
+template <typename T, std::enable_if_t<!is_complex<T>, int> = 0>
+void print_sparse_matrix(int m, int n, int* csrRowPtr, int* csrColInd, T* csrVal)
+{
+    printf("%d by %d matrix\n", m, n);
+
+    int idx = 0;
+    for(int i = 0; i < m; i++)
+    {
+        printf("    ");
+        for(int j = 0; j < n; j++)
+        {
+            if(idx < csrRowPtr[i + 1] && j == csrColInd[idx])
+            {
+                printf("%.8g, ", csrVal[idx]);
+                idx++;
+            }
+            else
+                printf("0, ");
+        }
+        printf("\n");
+    }
+
+    printf("\n");
+}
+
+/* ============================================================================================ */
+/*  read matrix or values from file */
+
+// integers:
+inline void
+    read_matrix(const std::string filenameS, const int m, const int n, int* A, const int lda)
+{
+    const char* filename = filenameS.c_str();
+    FILE*       mat;
+    mat = fopen(filename, "r");
+    int v;
+
+    if(mat == NULL)
+        throw std::invalid_argument(std::string("Error: Could not open file ") + filename
+                                    + " with test data...");
+
+    for(int j = 0; j < n; ++j)
+    {
+        for(int i = 0; i < m; ++i)
+        {
+            int read = fscanf(mat, "%d", &v);
+            if(read != 1)
+                throw std::out_of_range(std::string("Error: Could not read element from file ")
+                                        + filename);
+            A[i + j * lda] = v;
+        }
+    }
+
+    fclose(mat);
+}
+inline void read_last(const std::string filenameS, int* A)
+{
+    const char* filename = filenameS.c_str();
+    FILE*       mat;
+    mat = fopen(filename, "r");
+    int v;
+
+    if(mat == NULL)
+        throw std::invalid_argument(std::string("Error: Could not open file ") + filename
+                                    + " with test data...");
+
+    while(fscanf(mat, "%d", &v) == 1)
+    {
+        // do nothing
+    }
+
+    *A = v;
+}
+
+// singles:
+inline void
+    read_matrix(const std::string filenameS, const int m, const int n, float* A, const int lda)
+{
+    const char* filename = filenameS.c_str();
+    FILE*       mat;
+    mat = fopen(filename, "r");
+    float v;
+
+    if(mat == NULL)
+        throw std::invalid_argument(std::string("Error: Could not open file ") + filename
+                                    + " with test data...");
+
+    for(int j = 0; j < n; ++j)
+    {
+        for(int i = 0; i < m; ++i)
+        {
+            int read = fscanf(mat, "%g", &v);
+            if(read != 1)
+                throw std::out_of_range(std::string("Error: Could not read element from file ")
+                                        + filename);
+            A[i + j * lda] = v;
+        }
+    }
+
+    fclose(mat);
+}
+
+// doubles:
+inline void
+    read_matrix(const std::string filenameS, const int m, const int n, double* A, const int lda)
+{
+    const char* filename = filenameS.c_str();
+    FILE*       mat;
+    mat = fopen(filename, "r");
+    double v;
+
+    if(mat == NULL)
+        throw std::invalid_argument(std::string("Error: Could not open file ") + filename
+                                    + " with test data...");
+
+    for(int j = 0; j < n; ++j)
+    {
+        for(int i = 0; i < m; ++i)
+        {
+            int read = fscanf(mat, "%lg", &v);
+            if(read != 1)
+                throw std::out_of_range(std::string("Error: Could not read element from file ")
+                                        + filename);
+            A[i + j * lda] = v;
+        }
+    }
+
+    fclose(mat);
+}
+
+// complex float:
+inline void read_matrix(
+    const std::string filenameS, const int m, const int n, hipsolverComplex* A, const int lda)
+{
+    const char* filename = filenameS.c_str();
+    FILE*       mat;
+    mat = fopen(filename, "r");
+    float v;
+
+    if(mat == NULL)
+        throw std::invalid_argument(std::string("Error: Could not open file ") + filename
+                                    + " with test data...");
+
+    for(int j = 0; j < n; ++j)
+    {
+        for(int i = 0; i < m; ++i)
+        {
+            int read = fscanf(mat, "%g", &v);
+            if(read != 1)
+                throw std::out_of_range(std::string("Error: Could not read element from file ")
+                                        + filename);
+            A[i + j * lda] = {v, 0};
+        }
+    }
+
+    fclose(mat);
+}
+
+// complex double:
+inline void read_matrix(
+    const std::string filenameS, const int m, const int n, hipsolverDoubleComplex* A, const int lda)
+{
+    const char* filename = filenameS.c_str();
+    FILE*       mat;
+    mat = fopen(filename, "r");
+    double v;
+
+    if(mat == NULL)
+        throw std::invalid_argument(std::string("Error: Could not open file ") + filename
+                                    + " with test data...");
+
+    for(int j = 0; j < n; ++j)
+    {
+        for(int i = 0; i < m; ++i)
+        {
+            int read = fscanf(mat, "%lg", &v);
+            if(read != 1)
+                throw std::out_of_range(std::string("Error: Could not read element from file ")
+                                        + filename);
+            A[i + j * lda] = {v, 0};
+        }
+    }
+
+    fclose(mat);
+}
+
 /* =============================================================================================== */
 
 /* ============================================================================================ */
@@ -443,13 +789,6 @@ void print_matrix(T* CPU_result, T* GPU_result, int m, int n, int lda)
 extern "C" {
 #endif
 
-/* ============================================================================================ */
-/*  device query and print out their ID and name */
-int query_device_property();
-
-/*  set current device to device_id */
-void set_device(int device_id);
-
 /* ============================================================================================ */
 /*  timing: HIP only provides very limited timers function clock() and not general;
             hipsolver sync CPU and device and use more accurate CPU timer*/
