@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-# Copyright (C) 2022 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -45,15 +45,23 @@ function display_help()
   echo "    [-v|--rocm-dev] Set specific rocm-dev version"
   echo "    [-b|--rocblas] Set specific rocblas version"
   echo "    [--rocblas-path] Set specific path to custom built rocblas"
+  echo "    [--hipblas-path] Set specific path to custom built hipblas"
   echo "    [-s|--rocsolver] Set specific rocsolver version"
   echo "    [--rocsolver-path] Set specific path to custom built rocsolver"
+  echo "    [--rocsparse] Set specific rocsparse version"
+  echo "    [--rocsparse-path] Set specific path to custom built rocsparse"
+  echo "    [--hipsparse-path] Set specific path to custom built hipsparse"
   echo "    [--static] Create static library instead of shared library"
   echo "    [--codecoverage] Build with code coverage profiling enabled, excluding release mode."
   echo "    [--address-sanitizer] Build with address sanitizer enabled. Uses hipcc to compile"
+  echo "    [--sparse] Build with sparse functionality enabled at build time."
+  echo "    [--no-sparse] Build with sparse functionality tests disabled."
+  echo "    [--docs] (experimental) Pass this flag to build the documentation from source."
   echo "    [--cmake-arg] Forward the given argument to CMake when configuring the build"
-  echo "    [--rm-legacy-include-dir] Remove legacy include dir Packaging added for file/folder reorg backward compatibility."
 }
 
+# Find project root directory
+main=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
 
 # This function is helpful for dockerfiles that do not have sudo installed, but the default user is root
 # true is a system command that completes successfully, function returns success
@@ -132,6 +140,18 @@ install_dnf_packages( )
   done
 }
 
+# Take an array of packages as input, and install those packages with 'zypper' if they are not already installed
+install_zypper_packages( )
+{
+  package_dependencies=("$@")
+  for package in "${package_dependencies[@]}"; do
+    if [[ $(rpm -q ${package} &> /dev/null; echo $? ) -ne 0 ]]; then
+      printf "\033[32mInstalling \033[33m${package}\033[32m from distro package manager\033[0m\n"
+      elevate_if_not_root zypper -n --no-gpg-checks install ${package}
+    fi
+  done
+}
+
 # Take an array of packages as input, and delegate the work to the appropriate distro installer
 # prereq: ${ID} must be defined before calling
 # prereq: ${build_clients} must be defined before calling
@@ -159,57 +179,89 @@ install_packages( )
     library_dependencies_ubuntu+=( "cuda" )
     library_dependencies_centos+=( "" ) # how to install cuda on centos?
     library_dependencies_fedora+=( "" ) # how to install cuda on fedora?
-  elif [[ "${build_hip_clang}" == false ]]; then
-    # Custom rocm-dev installation
-    if [[ -z ${custom_rocm_dev+foo} ]]; then
-      # Install base rocm-dev package unless -v/--rocm-dev flag is passed
-      library_dependencies_ubuntu+=( "rocm-dev" )
-      library_dependencies_centos+=( "rocm-dev" )
-      library_dependencies_fedora+=( "rocm-dev" )
-      library_dependencies_sles+=( "rocm-dev" )
-    else
-      # Install rocm-specific rocm-dev package
-      library_dependencies_ubuntu+=( "${custom_rocm_dev}" )
-      library_dependencies_centos+=( "${custom_rocm_dev}" )
-      library_dependencies_fedora+=( "${custom_rocm_dev}" )
-      library_dependencies_sles+=( "${custom_rocm_dev}" )
+
+  else
+    if [[ "${build_sparse}" == true ]]; then
+      library_dependencies_ubuntu+=( "libsuitesparse-dev" )
+      library_dependencies_centos+=( "suitesparse-devel" )
+      library_dependencies_centos8+=( "suitesparse-devel" )
+      library_dependencies_fedora+=( "suitesparse-devel" )
+      library_dependencies_sles+=( "suitesparse-devel" )
     fi
 
-    # Custom rocblas installation
-    # Do not install rocblas if --rocblas_path flag is set,
-    # as we will be building against our own rocblas intead.
-    if [[ -z ${rocblas_path+foo} ]]; then
-      if [[ -z ${custom_rocblas+foo} ]]; then
-        # Install base rocblas package unless -b/--rocblas flag is passed
-        library_dependencies_ubuntu+=( "rocblas" )
-        library_dependencies_centos+=( "rocblas" )
-        library_dependencies_fedora+=( "rocblas" )
-        library_dependencies_sles+=( "rocblas" )
+    if [[ "${build_hip_clang}" == false ]]; then
+      # Custom rocm-dev installation
+      if [[ -z ${custom_rocm_dev+foo} ]]; then
+        # Install base rocm-dev package unless -v/--rocm-dev flag is passed
+        library_dependencies_ubuntu+=( "rocm-dev" )
+        library_dependencies_centos+=( "rocm-dev" )
+        library_dependencies_fedora+=( "rocm-dev" )
+        library_dependencies_sles+=( "rocm-dev" )
       else
-        # Install rocm-specific rocblas package
-        library_dependencies_ubuntu+=( "${custom_rocblas}" )
-        library_dependencies_centos+=( "${custom_rocblas}" )
-        library_dependencies_fedora+=( "${custom_rocblas}" )
-        library_dependencies_sles+=( "${custom_rocblas}" )
+        # Install rocm-specific rocm-dev package
+        library_dependencies_ubuntu+=( "${custom_rocm_dev}" )
+        library_dependencies_centos+=( "${custom_rocm_dev}" )
+        library_dependencies_fedora+=( "${custom_rocm_dev}" )
+        library_dependencies_sles+=( "${custom_rocm_dev}" )
       fi
-    fi
 
-    # Custom rocsolver installation
-    # Do not install rocsolver if --rocsolver_path flag is set,
-    # as we will be building against our own rocsolver intead.
-    if [[ -z ${rocsolver_path+foo} ]]; then
-      if [[ -z ${custom_rocsolver+foo} ]]; then
-        # Install base rocsolver package unless -s/--rocsolver flag is passed
-        library_dependencies_ubuntu+=( "rocsolver" )
-        library_dependencies_centos+=( "rocsolver" )
-        library_dependencies_fedora+=( "rocsolver" )
-        library_dependencies_sles+=( "rocsolver" )
-      else
-        # Install rocm-specific rocsolver package
-        library_dependencies_ubuntu+=( "${custom_rocsolver}" )
-        library_dependencies_centos+=( "${custom_rocsolver}" )
-        library_dependencies_fedora+=( "${custom_rocsolver}" )
-        library_dependencies_sles+=( "${custom_rocsolver}" )
+      # Custom rocblas installation
+      # Do not install rocblas if --rocblas_path flag is set,
+      # as we will be building against our own rocblas instead.
+      if [[ -z ${rocblas_path+foo} ]]; then
+        if [[ -z ${custom_rocblas+foo} ]]; then
+          # Install base rocblas package unless -b/--rocblas flag is passed
+          library_dependencies_ubuntu+=( "rocblas" )
+          library_dependencies_centos+=( "rocblas" )
+          library_dependencies_fedora+=( "rocblas" )
+          library_dependencies_sles+=( "rocblas" )
+        else
+          # Install rocm-specific rocblas package
+          library_dependencies_ubuntu+=( "${custom_rocblas}" )
+          library_dependencies_centos+=( "${custom_rocblas}" )
+          library_dependencies_fedora+=( "${custom_rocblas}" )
+          library_dependencies_sles+=( "${custom_rocblas}" )
+        fi
+      fi
+
+      # Custom rocsolver installation
+      # Do not install rocsolver if --rocsolver_path flag is set,
+      # as we will be building against our own rocsolver instead.
+      if [[ -z ${rocsolver_path+foo} ]]; then
+        if [[ -z ${custom_rocsolver+foo} ]]; then
+          # Install base rocsolver package unless -s/--rocsolver flag is passed
+          library_dependencies_ubuntu+=( "rocsolver" )
+          library_dependencies_centos+=( "rocsolver" )
+          library_dependencies_fedora+=( "rocsolver" )
+          library_dependencies_sles+=( "rocsolver" )
+        else
+          # Install rocm-specific rocsolver package
+          library_dependencies_ubuntu+=( "${custom_rocsolver}" )
+          library_dependencies_centos+=( "${custom_rocsolver}" )
+          library_dependencies_fedora+=( "${custom_rocsolver}" )
+          library_dependencies_sles+=( "${custom_rocsolver}" )
+        fi
+      fi
+
+      if [[ "${build_sparse}" == true ]]; then
+        # Custom rocsparse installation
+        # Do not install rocsparse if --rocsparse_path flag is set,
+        # as we will be building against our own rocsparse intead.
+        if [[ -z ${rocsparse_path+foo} ]]; then
+          if [[ -z ${custom_rocsparse+foo} ]]; then
+            # Install base rocsparse package unless --rocsparse flag is passed
+            library_dependencies_ubuntu+=( "rocsparse" )
+            library_dependencies_centos+=( "rocsparse" )
+            library_dependencies_fedora+=( "rocsparse" )
+            library_dependencies_sles+=( "rocsparse" )
+          else
+            # Install rocm-specific rocsparse package
+            library_dependencies_ubuntu+=( "${custom_rocsparse}" )
+            library_dependencies_centos+=( "${custom_rocsparse}" )
+            library_dependencies_fedora+=( "${custom_rocsparse}" )
+            library_dependencies_sles+=( "${custom_rocsparse}" )
+          fi
+        fi
       fi
     fi
   fi
@@ -271,18 +323,6 @@ install_packages( )
   esac
 }
 
-# Take an array of packages as input, and install those packages with 'zypper' if they are not already installed
-install_zypper_packages( )
-{
-  package_dependencies=("$@")
-  for package in "${package_dependencies[@]}"; do
-    if [[ $(rpm -q ${package} &> /dev/null; echo $? ) -ne 0 ]]; then
-      printf "\033[32mInstalling \033[33m${package}\033[32m from distro package manager\033[0m\n"
-      elevate_if_not_root zypper -n --no-gpg-checks install ${package}
-    fi
-  done
-}
-
 # given a relative path, returns the absolute path
 make_absolute_path( ) {
   (cd "$1" && pwd -P)
@@ -291,7 +331,7 @@ make_absolute_path( ) {
 # #################################################
 # Pre-requisites check
 # #################################################
-# Exit code 0: alls well
+# Exit code 0: all is well
 # Exit code 1: problems with getopt
 # Exit code 2: problems with supported platforms
 
@@ -324,6 +364,9 @@ build_cuda=false
 build_hip_clang=true
 build_release=true
 build_relocatable=false
+build_sparse=false
+build_sparse_tests=true
+build_docs=false
 cmake_prefix_path=/opt/rocm
 cuda_path=/usr/local/cuda
 rocm_path=/opt/rocm
@@ -332,7 +375,6 @@ build_static=false
 build_release_debug=false
 build_codecoverage=false
 build_address_sanitizer=false
-build_freorg_bkwdcomp=true
 declare -a cmake_common_options
 declare -a cmake_client_options
 
@@ -343,7 +385,7 @@ declare -a cmake_client_options
 # check if we have a modern version of getopt that can handle whitespace and long parameters
 getopt -T
 if [[ $? -eq 4 ]]; then
-  GETOPT_PARSE=$(getopt --name "${0}" --longoptions help,install,codecoverage,clients,no-solver,dependencies,debug,relwithdebinfo,hip-clang,no-hip-clang,compiler:,cuda,use-cuda,cudapath:,static,cmakepp,relocatable:,rocm-dev:,rocblas:,rocblas-path:,rocsolver:,rocsolver-path:,custom-target:,address-sanitizer,rm-legacy-include-dir,cmake-arg: --options rhicndgkp:v:b:s: -- "$@")
+  GETOPT_PARSE=$(getopt --name "${0}" --longoptions help,install,codecoverage,clients,no-solver,dependencies,debug,relwithdebinfo,hip-clang,no-hip-clang,compiler:,cuda,use-cuda,cudapath:,static,cmakepp,relocatable:,rocm-dev:,rocblas:,rocblas-path:,hipblas-path:,rocsolver:,rocsolver-path:,rocsparse:,rocsparse-path:,hipsparse-path:,custom-target:,docs,address-sanitizer,sparse,no-sparse,cmake-arg: --options rhicndgkp:v:b:s: -- "$@")
 else
   echo "Need a new version of getopt"
   exit 1
@@ -403,12 +445,18 @@ while true; do
     --static)
         build_static=true
         shift ;;
+    --docs)
+        build_docs=true
+        shift ;;
     --address-sanitizer)
         build_address_sanitizer=true
         compiler=hipcc
         shift ;;
-    --rm-legacy-include-dir)
-        build_freorg_bkwdcomp=false
+    --sparse)
+        build_sparse=true
+        shift ;;
+    --no-sparse)
+        build_sparse_tests=false
         shift ;;
     -p|--cmakepp)
         cmake_prefix_path=${2}
@@ -425,12 +473,24 @@ while true; do
     --rocblas-path)
         rocblas_path=${2}
         shift 2 ;;
+    --hipblas-path)
+        hipblas_path=${2}
+        shift 2 ;;
     -s|--rocsolver)
          custom_rocsolver=${2}
          shift 2;;
     --rocsolver-path)
         rocsolver_path=${2}
         shift 2 ;;
+    --rocsparse)
+         custom_rocsparse=${2}
+         shift 2;;
+    --rocsparse-path)
+        rocsparse_path=${2}
+        shift 2 ;;
+    --hipsparse-path)
+        hipsparse_path=${2}
+        shift 2 ;;
     --prefix)
         install_prefix=${2}
         shift 2 ;;
@@ -462,7 +522,9 @@ printf "\033[32mCreating project build directory in: \033[33m${build_dir}\033[0m
 # prep
 # #################################################
 # ensure a clean build environment
-if [[ "${build_release}" == true ]]; then
+if [[ "${build_docs}" == true ]]; then
+  rm -rf -- "${build_dir}/html"
+elif [[ "${build_release}" == true ]]; then
   rm -rf ${build_dir}/release
 elif [[ "${build_release_debug}" == true ]]; then
   rm -rf ${build_dir}/release-debug
@@ -474,9 +536,18 @@ fi
 if [[ -n "${rocblas_path+x}" ]]; then
   rocblas_path="$(make_absolute_path "${rocblas_path}")"
 fi
+if [[ -n "${hipblas_path+x}" ]]; then
+  hipblas_path="$(make_absolute_path "${hipblas_path}")"
+fi
 if [[ -n "${rocsolver_path+x}" ]]; then
   rocsolver_path="$(make_absolute_path "${rocsolver_path}")"
 fi
+if [[ -n "${rocsparse_path+x}" ]]; then
+  rocsparse_path="$(make_absolute_path "${rocsparse_path}")"
+fi
+if [[ -n "${hipsparse_path+x}" ]]; then
+  hipsparse_path="$(make_absolute_path "${hipsparse_path}")"
+fi
 
 # Default cmake executable is called cmake
 cmake_executable=cmake
@@ -514,6 +585,24 @@ pushd .
   # configure & build
   # #################################################
 
+mkdir -p "$build_dir"
+
+# build documentation
+if [[ "${build_docs}" == true ]]; then
+  set -eu
+  container_name="build_$(head -c 10 /dev/urandom | base32)"
+
+  docs_build_command='cp -r /mnt/hipsolver /home/docs/ && cd /home/docs/hipsolver/docs && python3 -m sphinx -T -E -b html -d _build/doctrees -D language=en . ../build/html'
+  docker build -t hipsolver:docs -f "$main/docs/Dockerfile" "$main/docs"
+  docker run -v "$main:/mnt/hipsolver:ro" --name "$container_name" hipsolver:docs /bin/sh -c "$docs_build_command"
+  docker cp "$container_name:/home/docs/hipsolver/build/html" "$main/build/html"
+  absolute_build_dir=$(make_absolute_path "$build_dir")
+  set +x
+  echo 'Documentation Built:'
+  echo "HTML: file://$absolute_build_dir/html/index.html"
+  exit
+fi
+
   if [[ "${build_static}" == true ]]; then
     if [[ "${build_cuda}" == true ]]; then
       printf "Static library not supported for CUDA backend.\n"
@@ -557,11 +646,26 @@ pushd .
     cmake_common_options+=("-DCUSTOM_ROCBLAS=${rocblas_path}")
   fi
 
+  # custom hipblas
+  if [[ ${hipblas_path+foo} ]]; then
+    cmake_common_options+=("-DCUSTOM_HIPBLAS=${hipblas_path} -DBUILD_HIPBLAS_TESTS=ON")
+  fi
+
   # custom rocsolver
   if [[ ${rocsolver_path+foo} ]]; then
     cmake_common_options+=("-DCUSTOM_ROCSOLVER=${rocsolver_path}")
   fi
 
+  # custom rocsparse
+  if [[ ${rocsparse_path+foo} ]]; then
+    cmake_common_options+=("-DCUSTOM_ROCSPARSE=${rocsparse_path}")
+  fi
+
+  # custom hipsparse
+  if [[ ${hipsparse_path+foo} ]]; then
+    cmake_common_options+=("-DCUSTOM_HIPSPARSE=${hipsparse_path}")
+  fi
+
   # code coverage
   if [[ "${build_codecoverage}" == true ]]; then
       if [[ "${build_release}" == true ]]; then
@@ -576,11 +680,13 @@ pushd .
     cmake_common_options+=("-DBUILD_ADDRESS_SANITIZER=ON")
   fi
 
-if [[ "${build_freorg_bkwdcomp}" == true ]]; then
-  cmake_common_options="${cmake_common_options} -DBUILD_FILE_REORG_BACKWARD_COMPATIBILITY=ON"
-  else
-  cmake_common_options="${cmake_common_options} -DBUILD_FILE_REORG_BACKWARD_COMPATIBILITY=OFF"
-fi
+  # build with sparse
+  if [[ "${build_sparse}" == true ]]; then
+    cmake_common_options+=("-DBUILD_WITH_SPARSE=ON")
+  fi
+  if [[ "${build_sparse_tests}" == false ]]; then
+    cmake_common_options+=("-DBUILD_HIPSPARSE_TESTS=OFF")
+  fi
 
   # Build library
   if [[ "${build_relocatable}" == true ]]; then
