@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2022 Advanced Micro Devices, Inc.
+ * Copyright (C) 2020-2024 Advanced Micro Devices, Inc.
  * ************************************************************************ */
 
 // HIP = Heterogeneous-compute Interface for Portability
@@ -27,6 +27,15 @@
 /* Defines functions with the hipsolverDn prefix. APIs match those from cuSOLVER but may
  * result in degraded rocSOLVER performance.
  */
-#include "internal/hipsolver-compat.h"
+#include "internal/hipsolver-dense.h"
+#include "internal/hipsolver-dense64.h"
+
+/* Defines functions and types with the hipsolverRf prefix. APIs match those from cuSOLVER.
+ */
+#include "internal/hipsolver-refactor.h"
+
+/* Defines functions and types with the hipsolverSp prefix. APIs match those from cuSOLVER.
+ */
+#include "internal/hipsolver-sparse.h"
 
 #endif // HIPSOLVER_H
